@@ -1,4 +1,4 @@
-# $Id: Segment.pm,v 1.5 2009/06/04 15:33:30 scottcain Exp $
+# $Id: Segment.pm,v 1.5 2009-06-04 15:33:30 scottcain Exp $
 
 =head1 NAME
 
@@ -282,7 +282,9 @@ sub new {
         warn "srcfeature_id:$srcfeature_id" if DEBUG;
 
 	###URGI Is it the right place to set it?
-        $factory->refclass_feature_id($srcfeature_id);
+           #but don't set it if creating a feature for a hit object
+        $factory->refclass_feature_id($srcfeature_id)
+             unless defined($target);
 
         if ( $landmark_feature_id == $srcfeature_id ) {
 
